@@ -94,7 +94,7 @@
   #:use-module (ice-9 regex)
   #:use-module (ice-9 pretty-print)
   #:re-export (*nyacc-version*)
-  #:version (1 07 5))
+  #:version (1 08 0))
 
 (define fh-cpp-defs
   (cond
@@ -177,6 +177,11 @@
 (define (sfstr fmt . args)
   (apply simple-format #f fmt args))
 
+(define (ppstr exp)
+  (call-with-output-string
+    (lambda (port)
+      (pretty-print exp port #:per-line-prefix "  "))))
+
 (define (make-arg-namer)
   (let ((ix 0))
     (lambda ()
@@ -460,12 +465,15 @@
 
       ;; bs does not support function pointers, but fh does now
       (((function-returning (param-list . ,params)) . ,tail)
+       ;;(sferr "\n--FUNCTION->bs-desc:\n")
        `(fh:function ,(mtail->bs-desc tail)
 		     (list ,@(gen-bs-decl-params params))))
       (((pointer-to) (function-returning (param-list . ,params)) . ,tail)
+       ;;(sferr "\np2FUNCTION->bs-desc:\n")
        `(fh:function ,(mtail->bs-desc tail)
 		     (list ,@(gen-bs-decl-params params))))
       (((pointer-to) (pointer-to) (function-returning . ,rest) . ,rest)
+       ;;(sferr "\np2FUNCTION->bs-desc:\n")
        `(fh:pointer 'void))
 
       (((pointer-to) (struct-ref . ,rest))
@@ -494,9 +502,8 @@
       (((extern) . ,rest) (mtail->bs-desc rest))
       
       (,otherwise
-       (sferr "mtail->bs-desc missed mdecl:\n")
-       (pperr mdecl-tail)
-       (fherr "mtail->bs-desc failed")))))
+       (fherr "mtail->bs-desc missed:\n~A" (ppstr mdecl-tail))))))
+
 
 
 ;; --- output routines ---------------
@@ -722,7 +729,8 @@
   (sfscm "(define-public (unwrap-~A n)\n" name)
   (sfscm "  (cond\n")
   (sfscm "   ((symbol? n)\n")
-  (sfscm "    (or (assq-ref ~A-enum-nvl n) (error \"bad arg\")))\n" name)
+  (sfscm "    (or (assq-ref ~A-enum-nvl n)\n" name)
+  (sfscm "        (throw 'ffi-help-error \"bad arg: ~~A\" n)))\n")
   (sfscm "   ((integer? n) n)\n")
   (sfscm "   (else (error \"bad arg\"))))\n")
   (sfscm "(define-public (wrap-~A v)\n" name)
@@ -912,8 +920,7 @@
      (mtail->ffi-desc `((union-def ,field-list))))
     
     (,otherwise
-     (sferr "mtail->ffi-desc missed:\n") (pperr mdecl-tail)
-     (fherr "mtail->ffi-desc missed: ~S" mdecl-tail))))
+     (fherr "mtail->ffi-desc missed:\n~A" (ppstr mdecl-tail)))))
 
 ;; Return a mdecl for the return type.  The variable is called @code{NAME}.
 (define (gen-decl-return udecl)
@@ -958,6 +965,10 @@
 	    (let* ((udecl1 (expand-typerefs param (*udict*) ffi-defined))
 		   (udecl1 (udecl-rem-type-qual udecl1))
 		   (mdecl (udecl->mdecl udecl1 #:namer namer)))
+              ;;(sferr "gen-bs-decl-params:\n") (pperr param)
+              ;;(sferr "  ffi: ~S\n" ffi-defined)
+              ;;(sferr "  def: ~S\n" (*defined*))
+              ;;(sferr "  wrp: ~S\n" (*wrapped*))
 	      (cons (mtail->bs-desc (cdr mdecl)) seed))))
       '() params))))
 
@@ -1070,8 +1081,8 @@
       (((array-of) . ,rest) 'unwrap~array)
 
       (,otherwise
-       (sferr "mdecl->fh-unwrapper missed:\n") (pperr mdecl)
-       (fherr "mdecl->fh-unwrapper missed: ~S" mdecl)))))
+       (fherr "mdecl->fh-unwrapper missed:\n~A" (ppstr (cadr mdecl)))))))
+
 
 (define (mdecl->fh-wrapper mdecl)
   (let ((wrapped (*wrapped*)) (defined (*defined*)))
@@ -1122,7 +1133,9 @@
 
       (((pointer-to) . ,otherwise) #f)
 
-      (,otherwise (fherr "mdecl->fh-wrapper missed: ~S" mdecl)))))
+      (,otherwise
+       (fherr "mdecl->fh-wrapper missed:\n~A" (ppstr (cadr mdecl)))))))
+
 
 ;; given list of name-unwrap pairs generate function arg names
 (define (gen-exec-arg-names params)
@@ -1897,9 +1910,7 @@
       ;; === missed =====================
 
       (,otherwise
-       (sferr "ffi-help/cnvt-udecl missed:\n")
-       (pretty-print-c99 clean-udecl)
-       (pperr clean-udecl)
+       (fherr "ffi-help/cnvt-udecl missed:\n~A" (ppstr clean-udecl))
        (values wrapped defined)))))
 
 
@@ -1926,7 +1937,7 @@
 	(defs
 	  (fold
 	   (lambda (name seed)
-	     (let* ((repl (expand-cpp-name name ddict))
+	     (let* ((repl (expand-cpp-name name ddict '()))
 		    (val (and (string? repl) (eval-code-string repl))))
 	       (if val
 		   (acons (string->symbol name) val seed)
